"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.bestZoomForBounds = bestZoomForBounds;

function _typeof(obj) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (obj) { return typeof obj; } : function (obj) { return obj && "function" == typeof Symbol && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }, _typeof(obj); }

/** A google.maps.LatLngBounds or object literal with lat and lng properties.
 *  @typedef {Object} LatLngBounds
 *  @property {number} lat
 *  @property {number} lng
 * /
/**
 * Determines the best zoom level for a Google map.
 * @param {LatLngBounds} bounds The bounds.
 * @param {Object} mapDimensions POJO with height and width properties.
 * @param {number} mapDimensions.height The height of the map's div.
 * @param {number} mapDimensions.width The width of the map's div.
 * @return {number} The best zoom level.
 */
function bestZoomForBounds(bounds, mapDimensions) {
  if (_typeof(bounds) !== "object" || _typeof(mapDimensions) !== "object" || !bounds || !mapDimensions) return null;
  var WORLD_DIM = {
    height: 256,
    width: 256
  };
  var ZOOM_MAX = 21;

  function latRad(lat) {
    var sin = Math.sin(lat * Math.PI / 180);
    var radX2 = Math.log((1 + sin) / (1 - sin)) / 2;
    return Math.max(Math.min(radX2, Math.PI), -Math.PI) / 2;
  }

  function zoom(mapPx, worldPx, fraction) {
    return Math.floor(Math.log(mapPx / worldPx / fraction) / Math.LN2);
  }

  var ne = bounds.getNorthEast();
  var sw = bounds.getSouthWest();
  var latFraction = (latRad(ne.lat()) - latRad(sw.lat())) / Math.PI;
  var lngDiff = ne.lng() - sw.lng();
  var lngFraction = (lngDiff < 0 ? lngDiff + 360 : lngDiff) / 360;
  var latZoom = zoom(mapDimensions.height, WORLD_DIM.height, latFraction);
  var lngZoom = zoom(mapDimensions.width, WORLD_DIM.width, lngFraction);
  return Math.min(latZoom, lngZoom, ZOOM_MAX);
}
//# sourceMappingURL=google-maps.js.map