"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.checkNthBitOn = checkNthBitOn;
exports.toggleNthBit = toggleNthBit;
exports.turnNthBitOff = turnNthBitOff;
exports.turnNthBitOn = turnNthBitOn;

var _CTypeError = _interopRequireDefault(require("../../modules/errors/CTypeError"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var throwIfInvalid = function throwIfInvalid(arg1, arg2) {
  if (typeof arg1 !== "number") throw new _CTypeError["default"]("number", arg1);
  if (typeof arg2 !== "number") throw new _CTypeError["default"]("number", arg2);
};
/**
 * Turn the nth bit off in a number.
 * @param {number} num 
 * @param {number} nth 
 * @returns {number} The value with nth bit off. 
 */


function turnNthBitOff(num, nth) {
  throwIfInvalid(num, nth);
  return num & ~(1 << nth - 1);
}
/**
 * Turn the nth bit on in a number.
 * @param {number} num 
 * @param {number} nth 
 * @returns {number} The value with nth bit on. 
 */


function turnNthBitOn(num, nth) {
  throwIfInvalid(num, nth);
  return num | 1 << nth - 1;
}
/**
 * Toggle the nth bit in a number.
 * @param {number} num 
 * @param {number} nth 
 * @returns {number} The value with nth bit toggled. 
 */


function toggleNthBit(num, nth) {
  throwIfInvalid(num, nth);
  return num ^ 1 << nth - 1;
}
/**
 * Check the nth bit is on in a number.
 * @param {number} num 
 * @param {number} nth 
 * @returns {boolean} True if the bit is on. 
 */


function checkNthBitOn(num, nth) {
  throwIfInvalid(num, nth);
  return num & 1 << nth - 1;
}
//# sourceMappingURL=bitwise.js.map