"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.binaryToDecimal = binaryToDecimal;
exports.convertBytes = convertBytes;
exports.decimalToBinary = decimalToBinary;
exports.toLocaleString = toLocaleString;

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _typeof(obj) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (obj) { return typeof obj; } : function (obj) { return obj && "function" == typeof Symbol && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }, _typeof(obj); }

/**
 * Convert a binary number to decimal.
 * @param {string} x The binary number to convert.
 * @returns The decimal number.
 */
function binaryToDecimal(x) {
  return parseInt(x, 2);
}
/**
 * Convert a decimal number to binary.
 * @param {number} x The decimal number to convert.
 * @returns The binary number.
 */


function decimalToBinary(x) {
  return x.toString(2);
}

var BYTE_UNITS = ["B", "kB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB"];
var BIBYTE_UNITS = ["B", "kiB", "MiB", "GiB", "TiB", "PiB", "EiB", "ZiB", "YiB"];
var BIT_UNITS = ["b", "kbit", "Mbit", "Gbit", "Tbit", "Pbit", "Ebit", "Zbit", "Ybit"];
var BIBIT_UNITS = ["b", "kibit", "Mibit", "Gibit", "Tibit", "Pibit", "Eibit", "Zibit", "Yibit"];
/**
 * Formats the given number using `Number.toLocaleString`.
 * - If locale is a string, the value is expected to be a locale-key (for example: `de`).
 * - If locale is true, the system default locale is used for translation.
 * - If no value for locale is specified, the number is returned unmodified.
 */

function toLocaleString(number, locale, options) {
  var result = number;

  if (typeof locale === "string" || Array.isArray(locale)) {
    result = number.toLocaleString(locale, options);
  } else if (locale === true || options !== undefined) {
    result = number.toLocaleString(undefined, options);
  }

  return result;
}
/**
 * Convert bytes to a human readable string: `1337 → 1.34 kB`
 * Useful for displaying file sizes for humans.
 * Note that it uses base-10 (e.g. kilobyte).
 * @see https://web.archive.org/web/20150324153922/https://pacoup.com/2009/05/26/kb-kb-kib-whats-up-with-that/
 * @param {number} number
 * @param {Object} options
 * @param {boolean} options.signed Default: false. Include plus sign for positive numbers.
 * If the difference is exactly zero a space character will be prepended instead for better
 * alignment.
 * @param {boolean} options.bits Default: false. Format the number as bits instead of bytes.
 * This can be useful when, for example, referring to bit rate.
 * @param {boolean} options.binary Default: false. Format the number using the Binary Prefix
 * instead of the SI Prefix. This can be useful for presenting memory amounts. However, this
 * should not be used for presenting file sizes.
 * @param {boolean | string} options.locale Default: false (No localization). Important: Only
 * the number and decimal separator are localized. The unit title is not and will not be
 * localized.
 * - If true: Localize the output using the system/browser locale.
 * - If string: Expects a BCP 47 language tag (For example: en, de, …)
 * - If string[]: Expects a list of BCP 47 language tags (For example: en, de, …)
 * @param {number} options.minimumFractionDigits Default: undefined. The minimum number of
 * fraction digits to display. If neither minimumFractionDigits or maximumFractionDigits are
 * set, the default behavior is to round to 3 significant digits.
 * @param {number} options.maximumFractionDigits Default: undefined. The maximum number of
 * digits to display. If neither minimumFractionDigits or maximumFractionDigits are set, the
 * default behavior is to round to 3 significant digits.
 */


function convertBytes(number, options) {
  if (!Number.isFinite(number)) throw new TypeError("Expected a finite number, got ".concat(_typeof(number), ": ").concat(number));
  options = _objectSpread({
    bits: false,
    binary: false
  }, options);
  var UNITS = options.bits ? options.binary ? BIBIT_UNITS : BIT_UNITS : options.binary ? BIBYTE_UNITS : BYTE_UNITS;
  if (options.signed && number === 0) return " 0 ".concat(UNITS[0]);
  var isNegative = number < 0;
  var prefix = isNegative ? "-" : options.signed ? "+" : "";
  if (isNegative) number = -number;
  var localeOptions;
  if (options.minimumFractionDigits !== undefined) localeOptions = {
    minimumFractionDigits: options.minimumFractionDigits
  };
  if (options.maximumFractionDigits !== undefined) localeOptions = _objectSpread({
    maximumFractionDigits: options.maximumFractionDigits
  }, localeOptions);

  if (number < 1) {
    var _numberString = toLocaleString(number, options.locale, localeOptions);

    return prefix + _numberString + " " + UNITS[0];
  }

  var exponent = Math.min(Math.floor(options.binary ? Math.log(number) / Math.log(1024) : Math.log10(number) / 3), UNITS.length - 1);
  number /= Math.pow(options.binary ? 1024 : 1000, exponent);
  if (!localeOptions) number = number.toPrecision(3);
  var numberString = toLocaleString(Number(number), options.locale, localeOptions);
  var unit = UNITS[exponent];
  return prefix + numberString + " " + unit;
}
//# sourceMappingURL=conversions.js.map