"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.randomFromRange = randomFromRange;
exports.randomHSLColor = randomHSLColor;
exports.randomNumber = randomNumber;
exports.randomValue = randomValue;
exports.shuffleArr = shuffleArr;
exports.uniqueRandom = uniqueRandom;
exports.uniqueRandomFromArr = uniqueRandomFromArr;

/**
 * Get a random number between min and max.
 * @param {number} min The minimum.
 * @param {number} max The maximum.
 * @returns {number} The random number in the range.
 */
function randomFromRange(min, max) {
  return Math.floor(Math.random() * (max - min + 1) + min);
}
/**
 * Get a random number up to num.
 * @param {number} num The maximum number.
 * @returns {number} The random number in the range of 0 - num.
 */


function randomNumber(num) {
  return Math.floor(Math.random() * num);
}
/**
 * Get a random value from an array.
 * @param {Array} arr The arr to retrieve a value from.
 * @returns {*} A random value from any index of the array.
 */


function randomValue(arr) {
  return arr[Math.floor(Math.random() * arr.length)];
}
/**
 * Shuffle the values of an array.
 * @param {Array} arr The arr to shuffle.
 * @returns {Array} The same array with it's values shuffled.
 */


function shuffleArr(array) {
  var original = JSON.stringify(original);
  var currentIndex = array.length,
      randomIndex;

  while (currentIndex !== 0) {
    randomIndex = Math.floor(Math.random() * currentIndex);
    currentIndex--;
    var _ref = [array[randomIndex], array[currentIndex]];
    array[currentIndex] = _ref[0];
    array[randomIndex] = _ref[1];
  }

  if (JSON.stringify(array) === original) return shuffleArr(array);
  return array;
}
/**
 * Generate a random HSL color.
 * @param {number} saturation The desired saturation.
 * @param {number} brightness The desired brightness.
 * @returns {string} A HSL color in the format: `hsl('color', 'saturation'%, 'brightness'%)`.
 */


function randomHSLColor() {
  var saturation = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 100;
  var brightness = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 100;
  return "hsl(".concat(Math.floor(Math.random() * 360), ", ").concat(saturation, "%, ").concat(brightness, "%)");
}
/**
 * Generate random numbers that are consecutively unique.
 * @param {number} minimum
 * @param {number} maximum
 * @returns {Function} a function, that when called, will return a random number that
 * is never the same as the previous (between minimum and maximum).
 */


function uniqueRandom(minimum, maximum) {
  var previousValue;
  return function random() {
    var number = Math.floor(Math.random() * (maximum - minimum + 1) + minimum);
    previousValue = number === previousValue && minimum !== maximum ? random() : number;
    return previousValue;
  };
}
/**
 * Get consecutively unique elements from an array.
 * @param {Array} arr
 * @returns {Function} a function, that when called, will return a random element
 * that's never the same as the previous.
 */


function uniqueRandomFromArr(arr) {
  var random = uniqueRandom(0, arr.length - 1);
  return function () {
    return arr[random()];
  };
}
//# sourceMappingURL=random.js.map