"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getDigits = getDigits;
exports.intSeparateThousands = intSeparateThousands;
exports.pluralize = pluralize;
exports.splitOnFirst = splitOnFirst;
exports.str = str;
exports.toDoubleQuotes = toDoubleQuotes;
exports.toSingleQuotes = toSingleQuotes;
exports.trim = trim;
exports.trimEnd = trimEnd;
exports.trimStart = trimStart;
exports.typeToStr = typeToStr;

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

/**
 * Get just the digits from a string.
 * @param {string} str the string to parse.
 * @returns {string} the string with all non-digits removed.
 */
function getDigits(str) {
  var asArr = _toConsumableArray(str);

  var res = [];
  asArr.forEach(function (v) {
    if (isNaN(+v)) return;
    res.push(v);
  });
  return res.join("");
}
/**
 * Given an integer, returns a string containing the same integer with additional thousands
 * separators.
 * @param {number} num An integer.
 * @param {string} [sep] The thousands separator to use.
 * @return {string} The same integer with thousands separators.
 */


function intSeparateThousands(num) {
  var sep = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : ",";
  return num.toString().split("").reverse().map(function (d, i, a) {
    return (i + 1) % 3 === 0 && i !== a.length - 1 ? "".concat(sep).concat(d) : d;
  }).reverse().join("");
}
/**
 * Casts a value to a string.
 * @param {string} v the value to cast.
 * @return {string} The string representation of the value.
 */


function str(v) {
  return "" + v;
}
/**
 * Trim characters from the beginning and end of a string.
 * @param {string} str the value to trim
 * @param {string} [trim] the character to trim. Defaults to " " (whitespace).
 * @param {Object} [options] object of options.
 * @param {boolean} [options.trimLeft] defaults to true if omitted
 * @param {boolean} [options.trimRight] defaults to true if omitted
 * @return {string} The trimmed string.
 */


function trim(str, trim) {
  var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  if (typeof trim === "undefined") trim = " ";
  trim = trim.replace(/[|\\{}()[\]^$+*?.]/g, "\\$&").replace(/-/g, "\\x2d");
  var left = typeof options.trimLeft === "undefined" || options.trimLeft;
  var right = typeof options.trimRight === "undefined" || options.trimRight;
  var parts = [];
  if (left) parts[parts.length] = "^[" + trim + "]+";
  if (right) parts[parts.length] = "[" + trim + "]+$";
  var regex = new RegExp(parts.join("|"), "gm");
  return str.replace(regex, "");
}
/**
 * Trim characters from the start of a string.
 * @param {string} str The string.
 * @param {string} mask Character to trim.
 * @return {string} The trimmed string.
 */


function trimStart(str, mask) {
  return trim(str, mask, {
    trimLeft: true,
    trimRight: false
  });
}
/**
 * Trim characters from the end of a string.
 * @param {string} str The string.
 * @param {string} characterMask Character to trim.
 * @return {string} The trimmed string.
 */


function trimEnd(str, characterMask) {
  return trim(str, characterMask, {
    trimLeft: false,
    trimRight: true
  });
}
/**
 * Converts a type to its string representation.
 * @param {*} type
 * @return {string} The string representation of `type`.
 */


function typeToStr(type) {
  return Object.prototype.toString.call(type);
}
/**
 * Pluralizes a word.
 */


function pluralize(word, count) {
  return count === 1 ? word : "".concat(word, "s");
}
/**
 * Split a string on the first occurrence of a given separator.
 * @param string - The string to split.
 * @param separator - The separator to split on.
 */


function splitOnFirst(string, separator) {
  if (!(typeof string === "string" && typeof separator === "string")) throw new TypeError("Expected the arguments to be of type `string`");
  if (string === "" || separator === "") return [];
  var separatorIndex = string.indexOf(separator);
  if (separatorIndex === -1) return [];
  return [string.slice(0, separatorIndex), string.slice(separatorIndex + separator.length)];
}

function transformQuotes(input, toSingleQuotes) {
  var result = "";
  var isBetweenQuotes = false;
  var quoteCharacter;
  var changeTo = "";
  var toChange = "";

  if (toSingleQuotes) {
    changeTo = "'";
    toChange = '"';
  } else {
    changeTo = '"';
    toChange = "'";
  }

  for (var i = 0; i < input.length; i++) {
    var current = input[i];
    var next = input[i + 1]; // Found double-quote or single-quote

    if (current === '"' || current === "'") {
      // If not processing in between quotes
      if (!isBetweenQuotes) {
        quoteCharacter = current;
        isBetweenQuotes = true;
        result += changeTo;
      } else if (quoteCharacter === current) {
        // If processing between quotes, close quotes
        result += changeTo;
        isBetweenQuotes = false;
      } else result += "\\" + changeTo; // Still inside quotes

    } else if (current === "\\" && (next === "'" || next === '"')) {
      // If escape character is found and double or single quote after
      // Escape + quote to change to
      if (next === changeTo) {
        // If in between quotes and quote is equal to changeTo only escape once
        result += isBetweenQuotes && quoteCharacter === changeTo ? "\\" + changeTo : "\\\\" + changeTo;
        i++;
      } else if (next === toChange) {
        // Escape + quote to be changed
        // If between quotes can mantain tochange
        result += isBetweenQuotes ? toChange : changeTo;
        i++;
      } else result += current;
    } else if (current === "\\" && next === "\\") {
      // Don't touch backslashes
      result += "\\\\";
      i++;
    } else result += current;
  }

  return result;
}
/**
 * Convert matching double-quotes to single-quotes: I "love" unicorns → I 'love' unicorns.
 */


function toSingleQuotes(string) {
  return transformQuotes(string, true);
}
/**
 * Convert matching single-quotes to double-quotes: I 'love' unicorns → I "love" unicorns.
 */


function toDoubleQuotes(string) {
  return transformQuotes(string, false);
}
//# sourceMappingURL=strings.js.map