"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.arrayMoveImmutable = arrayMoveImmutable;
exports.arrayMoveMutable = arrayMoveMutable;
exports.distinct = distinct;
exports.firstItem = firstItem;
exports.firstNested = firstNested;
exports.indexWrappedArray = indexWrappedArray;
exports.isArrayOfNumber = isArrayOfNumber;
exports.lastItem = lastItem;
exports.lastNested = lastNested;
exports.maxOf = maxOf;
exports.minOf = minOf;
exports.removeNulls = removeNulls;
exports.sortNums = sortNums;
exports.sumOf = sumOf;
exports.unshift = unshift;

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _iterableToArrayLimit(arr, i) { var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]; if (_i == null) return; var _arr = []; var _n = true; var _d = false; var _s, _e; try { for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function _typeof(obj) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (obj) { return typeof obj; } : function (obj) { return obj && "function" == typeof Symbol && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }, _typeof(obj); }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

/**
 * @param {any[]} arr an array of values.
 * @returns the distinct elements from the array. */
function distinct(arr) {
  return _toConsumableArray(new Set(arr));
}
/**
 * @param {number[]} arr an array of numbers.
 * @returns the sum of the array. */


function sumOf(arr) {
  return arr.reduce(function (a, b) {
    return a + b;
  }, 0);
}
/**
 * Check if an array is all numbers.
 * @param {any[]} arr an array of values.
 * @returns True if all the items in the array are numbers, or the array is empty. */


function isArrayOfNumber(arr) {
  return !arr.some(function (v) {
    return isNaN(v);
  });
}
/**
 * Get the first item in an array.
 * @param {any[]} arr an array of values.
 * @returns the first item in the array. */


function firstItem(arr) {
  return arr[0];
}
/**
 * Get the last item in an array.
 * @param {any[]} arr an array of values.
 * @returns the last item in the array. */


function lastItem(arr) {
  return arr[arr.length - 1];
}
/**
 * Get the first nested object in an array, if present.
 * @param {any[]} arr an array of values.
 * @returns {Object | undefined} the first nested array or object in the array. */


function firstNested(arr) {
  var nested;
  nested = arr.find(function (val) {
    return _typeof(val) === "object";
  });
  return nested;
}
/**
 * Get the last nested object in an array, if present.
 * @param {any[]} arr an array of values.
 * @returns the first nested array or object in the array. */


function lastNested(arr) {
  return firstNested(arr.reverse());
}
/**
 * Get the smallest value of an array. WARNING: may
 * return unpredictable results with non-int arrays.
 * @param {any[]} arr an array of values.
 * @returns {any} the smallest value in the array. */


function minOf(arr) {
  if (!arr.length) return void 0;
  return arr.reduce(function (currentMin, val) {
    return currentMin < val ? currentMin : val;
  }, +Infinity);
}
/**
 * Get the largeste value of an array. WARNING: may
 * return unpredictable results with non-int arrays.
 * @param {any[]} arr an array of values.
 * @returns {any} the largest value in the array. */


function maxOf(arr) {
  if (!arr.length) return void 0;
  return arr.reduce(function (currentMax, val) {
    return currentMax > val ? currentMax : val;
  }, -Infinity);
}
/**
 * Sort an array of numbers in ascending or descending order.
 * @param {number[]} arr an array of numbers.
 * @param {boolean} desc flag to sort ascending/descending. Default is ascending.
 * @returns {number[]} a sorted array without mutating the original. */


function sortNums(arr) {
  var desc = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
  return _toConsumableArray(arr).sort(function (a, b) {
    return desc ? b - a : a - b;
  });
}
/**
 * Remove nulls from an array. Returns a new array.
 * @param {any[]} arr
 * @returns an array without nulls that leaves the original unchanged. */


function removeNulls(arr) {
  var returnArr = [];
  arr.forEach(function (val) {
    if (val !== undefined && val !== null) returnArr.push(val);
  });
  return returnArr;
}
/**
 * 'Unshift' an item to an array (will become the new first item).
 * @param {any[]} arr The array to unshift.
 * @param {*} item the item to "prepend" to the array. */


function unshift(arr, item) {
  var len = arr.length;

  while (len) {
    arr[len] = arr[len - 1];
    len--;
  }

  arr[0] = item;
}
/**
 * Create an array which allows the usage of negative indices to retrieve its data,
 * where -1 indicates the last element of the array (same as `array.length - 1`). Can also
 * be configured to wrap indexes if the index is out of range in either direction.
 * @param {Array} array An array to use as the target object.
 * @param {Object} options Options.
 * @param {boolean} [options.wrapOverflow] Whether or not to wrap if the negative index is greater
 * than the length of the array. If false, undefined will be returned if the index is out of
 * range.
 * @return {Proxy} The proxy object for the given array.
 */


function indexWrappedArray(array) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {
    wrapOverflow: true
  };
  return new Proxy(array, {
    get: function get(arr, index) {
      index = Number(index);
      index = index < 0 ? index + arr.length : index;

      if (options.wrapOverflow) {
        while (index < 0) {
          index = index + arr.length;
        }

        while (index > arr.length) {
          index = index - arr.length;
        }

        if (index === arr.length) index = 0;
      }

      return arr[index];
    }
  });
}
/**
 * Moves the item to the new position in the array array. Useful for huge arrays where
 * absolute performance is needed.
 * @param {Array<any>} array
 * @param {number} fromIndex The index of the item to move. If negative, it will begin that
 * many elements from the end.
 * @param {number} toIndex The index of where to move the item. If negative, it will begin
 * that many elements from the end.
 * @returns {Array<any>} The same array with the item in it's new position.
 */


function arrayMoveMutable(array, fromIndex, toIndex) {
  var startIndex = fromIndex < 0 ? array.length + fromIndex : fromIndex;

  if (startIndex >= 0 && startIndex < array.length) {
    var endIndex = toIndex < 0 ? array.length + toIndex : toIndex;

    var _array$splice = array.splice(fromIndex, 1),
        _array$splice2 = _slicedToArray(_array$splice, 1),
        item = _array$splice2[0];

    array.splice(endIndex, 0, item);
  }
}
/**
 * Clones the given array, moves the item to a new position in the new array, and then
 * returns the new array. The given array is not mutated.
 * @param {Array<any>} array
 * @param {number} fromIndex The index of the item to move. If negative, it will begin that
 * many elements from the end.
 * @param {number} toIndex The index of where to move the item. If negative, it will begin
 * that many elements from the end.
 * @returns {Array<any>} A new array with the item in it's new position.
 */


function arrayMoveImmutable(array, fromIndex, toIndex) {
  var newArray = _toConsumableArray(array);

  arrayMoveMutable(newArray, fromIndex, toIndex);
  return newArray;
}
//# sourceMappingURL=arrays.js.map