"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.cloneArr = cloneArr;
exports.cloneDeep = cloneDeep;
exports.cloneRegExp = cloneRegExp;
exports.deepObjectCloningExtend = deepObjectCloningExtend;
exports.deepObjectExtend = deepObjectExtend;
exports.extend = extend;
exports.extendDecorate = extendDecorate;
exports.extendShallow = extendShallow;

var _strings = require("../misc/strings");

var _introspection = require("./introspection");

var _nestedProperties = require("./nested-properties");

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _iterableToArrayLimit(arr, i) { var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]; if (_i == null) return; var _arr = []; var _n = true; var _d = false; var _s, _e; try { for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function _createForOfIteratorHelper(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (!it) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e2) { throw _e2; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = it.call(o); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e3) { didErr = true; err = _e3; }, f: function f() { try { if (!normalCompletion && it["return"] != null) it["return"](); } finally { if (didErr) throw err; } } }; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

/** 
 * Shallowly clone an array. If deep cloning is needed, use 'cloneObjDeep'.
 * @param {Array} arr array to clone.
 * @returns {Array} a shallow copy of the array. */
function cloneArr(arr) {
  return arr.slice(0);
}
/**
 * Clones an object deeply, using either JSON or reflection, depending on whether
 * complex: true is passed in the options.
 * @param {Object} obj The object to clone.
 * @param {Object} [options] Optional extra options.
 * @param {boolean} [options.complex] Flag to say whether the value is a complex object (i.e 
 * a custom data type rather than simple nested objects and arrays). Defaults to false (will
 * use JSON to clone the object. This is usually faster and cheaper.)
 * @return {Object} The cloned object.
 */


function cloneDeep(obj) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {
    complex: false
  };
  if (!(options && options.complex)) return JSON.parse(JSON.stringify(obj));
  var newObject = new obj.constructor();

  var error = function error(msg, prop, property) {
    throw new Error(msg + prop + "' (" + (0, _strings.typeToStr)(property) + ") in object (" + (0, _strings.typeToStr)(obj) + ")");
  };

  for (var prop in obj) {
    // If the property is defined on the prototype, ignore it.
    // We don't want to assign it for each clone instance.
    if (!Object.prototype.hasOwnProperty.call(obj, prop)) continue;
    var property = obj[prop];
    if ((0, _introspection.isPrimitiveType)(property)) newObject[prop] = property;else if ((0, _introspection.isReferenceType)(property)) {
      if ((0, _introspection.hasCyclicReference)(property)) error("Circular reference detected '", prop, property);else {
        var clone = cloneDeep(property);
        newObject[prop] = clone;
      }
    } else error("Unknown type for property '", prop, property);
  }

  return newObject;
}
/**
 * Extends a destination object with the provided source objects.
 * @param {Object} destinationObj The destination object.
 * @param {...Object|Array} sourceObjects The source objects. If the last argument is an array
 * containing one single truthy element, it will be treated as an options parameter and its
 * single first truthy element will be treated as object containing the options for the extension.
 * The currently available options are:
 * - extendsArrays (boolean: false): Whether or not to extend nested arrays too (defaults to
 * false)
 * @return {Object} The destination object "destinationObj" given as parameter after extension.
 */


function extend(destinationObj) {
  var options = {};

  for (var _len = arguments.length, sourceObjects = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    sourceObjects[_key - 1] = arguments[_key];
  }

  if (sourceObjects.length) {
    var last = sourceObjects.pop();

    if ((0, _introspection.isArray)(last) && last.length === 1 && (0, _introspection.isObjectLiteral)(last[0])) {
      options = last[0];
    } else sourceObjects.push(last);
  }

  for (var _i = 0, _sourceObjects = sourceObjects; _i < _sourceObjects.length; _i++) {
    var sourceObject = _sourceObjects[_i];
    deepObjectExtend(destinationObj, sourceObject, options);
  }

  return destinationObj;
}
/**
 * Shallowly extends a destination object with the provided source objects (first dimension).
 * @param {Object} obj The destination object.
 * @param {...Object} args The source objects.
 * @return {Object} The destination object "destinationObject" given as parameter after shallow
 * extension.
 */


function extendShallow(obj) {
  for (var _len2 = arguments.length, args = new Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
    args[_key2 - 1] = arguments[_key2];
  }

  args.map(function (o) {
    return Object.keys(o).map(function (k) {
      return obj[k] = o[k];
    });
  });
  return obj;
}
/**
 * Deep object extension implementation.
 * Nothing is returned, but the destination object will be modified and merged with the source
 * object so that properties of the source object which are objects will recursively merge with
 * the corresponding destination property while the other properties with all the other types
 * will replace the properties of the destination object. Note: this method should not be used
 * for inheritance via the Prototypal Combination Inheritance pattern. Note: this method doesn't
 * perform a deep object cloning, it just extends the destinationObject by adding properties it
 * doesn't have in a deep way.
 * @param {Object} obj The destination object which will be modified and merged
 * with the source object.
 * @param {Object} fromObj The source object which will be used to extend the destination
 * object.
 * @param {Object} [extendArr] True if arrays should also be extended.
 * @return {undefined}
 */


function deepObjectExtend(obj, fromObj) {
  var extendArr = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;

  for (var prop in fromObj) {
    if (fromObj[prop] && (0, _introspection.isObjectLiteral)(fromObj[prop])) {
      obj[prop] = obj[prop] || {};
      deepObjectExtend(obj[prop], fromObj[prop], extendArr);
    } else if (extendArr && fromObj[prop] && (0, _introspection.isArray)(fromObj[prop])) {
      obj[prop] = obj[prop] || [];
      deepObjectExtend(obj[prop], fromObj[prop], extendArr);
    } else obj[prop] = fromObj[prop];
  }
}
/**
 * Deep object cloning extension implementation. If the source objects contain a property with
 * a reference type, a clone object of the same type of that property will be created and then
 * merged with the property object of the destination object.
 * @param {Object} obj The destination object which will be modified and merged
 * with the source object.
 * @param {...Object} args One or more objects which will be used to extend the 
 * destination object.
 * @return {undefined}
 */


function deepObjectCloningExtend(obj) {
  for (var i = 0; i < (arguments.length <= 1 ? 0 : arguments.length - 1); i++) {
    var source = i + 1 < 1 || arguments.length <= i + 1 ? undefined : arguments[i + 1];

    for (var prop in source) {
      if (source[prop] && (0, _introspection.isObjectLiteral)(source[prop])) {
        obj[prop] = obj[prop] || {};
        deepObjectExtend(obj[prop], cloneDeep(source[prop]));
      } else obj[prop] = source[prop];
    }
  }
}
/**
 * Extends a destination object with the provided source objects.
 * @param {Object} obj The destination object.
 * @param {...*} args The source objects with the last parameter being an array of rules,
 * each rule being a tuple array where the first element is an array of "ORed" property names
 * (strings or numbers) or regexes matching property names for which the corresponding values
 * should be decorated (or a single property name or regex matching a property name if the
 * decoration should only happen on a single property), and where the second element is a
 * callback to execute for each value which is a value of a property of a source object.
 * The callback has the following signature:
 * (value: *, prop: string|number, parent: Object) => *|undefined
 * The callback will receive the final value after extension, its associated property and 
 * the parent object where that property is set with that value.
 * Its returned value will be used as the final value of the property for that object in 
 * "destinationObject". If the last parameter is not an array of rules, it will be treated as 
 * the last source object to use for the extension the "extend" function will be simply called 
 * under the hood without any decoration logic).
 * @return {Object} The destination object "destinationObject" given as parameter after extension
 * and, if the callback is given as the last parameter, after applying the given callback.
 */


function extendDecorate(obj) {
  for (var _len3 = arguments.length, args = new Array(_len3 > 1 ? _len3 - 1 : 0), _key3 = 1; _key3 < _len3; _key3++) {
    args[_key3 - 1] = arguments[_key3];
  }

  var rules = args[args.length - 1];
  if (!(0, _introspection.isArray)(rules)) return extend.apply(void 0, [obj].concat(args));
  args.pop();
  var sourceObjects = args;
  var initialRetValue = {};
  var matchedRulesMap = new Map();
  var callbacksMap = new Map();
  var paths = [];

  var mapKeys = function mapKeys(obj, sourceObj, currStack, currPath) {
    for (var key in sourceObj) {
      currStack.push({
        obj: obj,
        sourceObj: sourceObj,
        prop: key,
        path: [].concat(_toConsumableArray(currPath), [key])
      });
    }
  };

  var matchRule = function matchRule(rule, prop) {
    if (rule instanceof RegExp && typeof prop === "string") return prop.match(rule);
    return rule === prop;
  };

  var matchArrayRule = function matchArrayRule(arrayRule, prop) {
    var _iterator = _createForOfIteratorHelper(arrayRule),
        _step;

    try {
      for (_iterator.s(); !(_step = _iterator.n()).done;) {
        var rule = _step.value;
        if (matchRule(rule, prop)) return true;
      }
    } catch (err) {
      _iterator.e(err);
    } finally {
      _iterator.f();
    }

    return false;
  };

  var ruleMatches = function ruleMatches(rule, prop) {
    if ((0, _introspection.isArray)(rule)) return matchArrayRule(rule, prop);
    return matchRule(rule, prop);
  };

  var matchRules = function matchRules(prop) {
    if (!matchedRulesMap.has(prop)) {
      var callbacks = [];

      var _iterator2 = _createForOfIteratorHelper(rules),
          _step2;

      try {
        for (_iterator2.s(); !(_step2 = _iterator2.n()).done;) {
          var _step2$value = _slicedToArray(_step2.value, 2),
              rule = _step2$value[0],
              callback = _step2$value[1];

          if (ruleMatches(rule, prop)) callbacks.push(callback);
        }
      } catch (err) {
        _iterator2.e(err);
      } finally {
        _iterator2.f();
      }

      matchedRulesMap.set(prop, callbacks);
    }

    return matchedRulesMap.get(prop);
  };

  for (var _i2 = 0, _sourceObjects2 = sourceObjects; _i2 < _sourceObjects2.length; _i2++) {
    var sourceObject = _sourceObjects2[_i2];
    var currStack = [];
    var currentPath = [];
    mapKeys(obj, sourceObject, currStack, currentPath);

    var _loop = function _loop() {
      var _currStack$pop = currStack.pop(),
          obj = _currStack$pop.obj,
          sourceObj = _currStack$pop.sourceObj,
          prop = _currStack$pop.prop,
          currPath = _currStack$pop.currPath;

      if (sourceObj[prop] && (0, _introspection.isObjectLiteral)(sourceObj[prop])) {
        obj[prop] = (0, _introspection.isObjectLiteral)(obj[prop]) ? obj[prop] : {};
        mapKeys(obj[prop], sourceObj[prop], currStack, currPath);
      } else obj[prop] = sourceObj[prop];

      var callbacks = matchRules(prop);

      if (callbacks && callbacks.length) {
        if (!(0, _nestedProperties.nestedTreeMapHas)(callbacksMap, currPath)) paths.push(_toConsumableArray(currPath));
        (0, _nestedProperties.nestedTreeMapSet)(callbacksMap, currPath, callbacks.map(function (callback) {
          return function (retValue) {
            return function (path) {
              var value = retValue === initialRetValue ? obj[prop] : retValue;
              return callback(value, prop, obj, path);
            };
          };
        }));
      }
    };

    while (currStack.length) {
      _loop();
    }
  } // Decorating the final nested values.


  for (var i = paths.length - 1; i >= 0; i--) {
    var path = paths[i];
    var callbacks = (0, _nestedProperties.nestedTreeMapGet)(callbacksMap, path) || [];
    var retValue = initialRetValue;

    var _iterator3 = _createForOfIteratorHelper(callbacks),
        _step3;

    try {
      for (_iterator3.s(); !(_step3 = _iterator3.n()).done;) {
        var callback = _step3.value;
        retValue = callback(retValue)(path);
      }
    } catch (err) {
      _iterator3.e(err);
    } finally {
      _iterator3.f();
    }

    if (retValue !== initialRetValue) setNestedPropertyValue(obj, path, retValue);
  }

  return obj;
}

var regExpFlagMap = {
  global: "g",
  ignoreCase: "i",
  multiline: "m",
  dotAll: "s",
  sticky: "y",
  unicode: "u"
};
/**
 * Clone a RegExp.
 * @param {RegExp} regexp 
 * @param {Object} options Optionally modify the cloned RegExp instance.
    Properties: source, global, ignoreCase, multiline, dotAll, sticky, unicode, lastIndex
 * @returns The cloned instance, configured with any passed options.
 */

function cloneRegExp(regexp) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  if (!regexp instanceof RegExp) {
    throw new TypeError("Expected a RegExp instance");
  }

  var flags = Object.keys(regExpFlagMap).map(function (flag) {
    return (typeof options[flag] === "boolean" ? options[flag] : regexp[flag]) ? regExpFlagMap[flag] : "";
  }).join("");
  var clonedRegexp = new RegExp(options.source || regexp.source, flags);
  clonedRegexp.lastIndex = typeof options.lastIndex === "number" ? options.lastIndex : regexp.lastIndex;
  return clonedRegexp;
}
//# sourceMappingURL=clone.js.map