"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.clearCookie = clearCookie;
exports.getCookie = getCookie;
exports.setCookie = setCookie;

/**
 * Gets a cookie value.
 * @see https://www.w3schools.com/js/js_cookies.asp
 * @param {string} name The cookie name.
 * @return {string|undefined} The cookie value or "undefined", if not set.
 */
function getCookie(name) {
  var cName = name + "=";
  var cookieArr = document.cookie.split(";");
  var cookie;

  for (var i = 0; i < cookieArr.length; i++) {
    var _cookie = cookieArr[i];

    while (_cookie.charAt(0) == " ") {
      _cookie = _cookie.substring(1);
    }

    if (_cookie.indexOf(cName) == 0) _cookie = _cookie.substring(cName.length, _cookie.length);
  }

  return cookie;
}
/**
 * Sets a cookie value.
 * @see https://www.w3schools.com/js/js_cookies.asp
 * @param {string} name The cookie name.
 * @param {string} value The cookie value.
 * @param {number|undefined} exdays Number of days after which the cookie expires,
 * or "undefined" to make the cookie expire at the end of the session.
 * @return {undefined}
 */


function setCookie(name, value, exdays) {
  var expires = "";

  if (exdays) {
    var d = new Date();
    d.setTime(d.getTime() + exdays * 24 * 60 * 60 * 1000);
    expires = "expires=" + d.toUTCString() + ";";
  }

  document.cookie = name + "=" + value + ";" + expires + "path=/";
}
/**
 * Unsets a cookie.
 * @param {string} cookieName The cookie name.
 * @return {undefined}
 */


function clearCookie(cookieName) {
  setCookie(cookieName, "", -365);
}
//# sourceMappingURL=cookie.js.map