"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.buildFormData = buildFormData;
exports.objToFormData = objToFormData;

function _typeof(obj) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (obj) { return typeof obj; } : function (obj) { return obj && "function" == typeof Symbol && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }, _typeof(obj); }

/**
 * Builds a form data instance or object recursively.
 * @see https://stackoverflow.com/questions/22783108/convert-js-object-to-form-data#answer-42483509
 * @param {FormData|Object} formData Form data instance or JS POJO object.
 * @param {Array|Object} data JS POJO object or array with the form data to use to build the
 * forms data.
 * @param {boolean} [encodeParams] Optionally instructs the function if the parameters
 * should be encoded (keys and values) using "encodeURIComponent".
 * @param {string} [parentKey] Parent key for nested parameters (used internally for recursion).
 * @return {undefined}
 */
function buildFormData(formData, data, encodeParams, parentKey) {
  if (data && _typeof(data) === "object" && !(data instanceof Date) && !(data instanceof File)) {
    Object.keys(data).forEach(function (key) {
      buildFormData(formData, data[key], encodeParams, parentKey ? "".concat(parentKey, "[").concat(key, "]") : encodeParams ? encodeURIComponent(key) : key);
    });
  } else {
    // Leaf value.
    var value = data == null ? "" : encodeParams ? encodeURIComponent(data) : data;
    formData instanceof FormData ? formData.append(parentKey, value) : formData[parentKey] = value;
  }
}
/**
 * Converts an object (array or JS POJO object) to a form data instance.
 * @param {Array|Object} data The data (array or POJO object).
 * @return {FormData} A form data instance.
 */


function objToFormData(data) {
  var formData = new FormData();
  buildFormData(formData, data);
  return formData;
}
//# sourceMappingURL=form-data.js.map